/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Some useful functions */

#include "radius_gw.h"

#include <stdlib.h>

void rg_list_init(struct rg_list * plist)
{
	ASSERT(plist != NULL);
	plist->next = plist;
	plist->prev = plist;
	plist->head = plist;
}

int rg_list_is_empty(struct rg_list * plist)
{
	ASSERT(plist != NULL);
	return plist == plist->next;
}

void rg_list_insert_after(struct rg_list * ref, struct rg_list * item)
{
	ASSERT(ref != NULL);
	ASSERT(item != NULL);
	ASSERT(rg_list_is_empty(item));
	item->next = ref->next;
	item->prev = ref;
	item->head = ref->head;
	ref->next->prev = item;
	ref->next = item;
}

void rg_list_insert_before(struct rg_list * ref, struct rg_list * item)
{
	ASSERT(ref != NULL);
	ASSERT(item != NULL);
	ASSERT(rg_list_is_empty(item));
	item->prev = ref->prev;
	item->next = ref;
	item->head = ref->head;
	ref->prev->next = item;
	ref->prev = item;
}

void rg_list_unlink(struct rg_list * plist)
{
	ASSERT(plist != NULL);
	if (plist->head == plist)
		return;
	plist->next->prev = plist->prev;
	plist->prev->next = plist->next;
	plist->next = plist;
	plist->prev = plist;
	plist->head = plist;
}


void rg_msg_free(rad_t * msg)
{
	while ( ! rg_list_is_empty( &msg->attributes ) ) {
		struct rg_list * item = msg->attributes.next;
		rg_list_unlink(item);
		free(item);
	}
	free(msg);	
}
