/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Manage incoming RADIUS message. */

#include "radius_gw_internal.h"

/* How many threads to work on messages ? */
#define NB_WORKERS	2


static pthread_mutex_t work_mtx = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t work_cond = PTHREAD_COND_INITIALIZER;

static struct rg_list work_data;
struct work_item {
	struct rg_list chain;
	rad_t * msg;
	void * cli;
};

static pthread_t workers[NB_WORKERS];

static void * work_th(void * arg)
{
	TRACE_ENTRY("%p", arg);
	
	THREAD_NAME("worker");
	
	CHECK_POSIX_DO( pthread_mutex_lock(&work_mtx), return NULL );
	
in:
	/* wait for new work data */
	if (rg_list_is_empty(&work_data)) {
		CHECK_POSIX_DO( pthread_cond_wait( &work_cond, &work_mtx ), goto out );
		goto in;
	}
	
	
	
	/* process the data */
	
	

out:
	CHECK_POSIX_DO( pthread_mutex_unlock(&work_mtx), );
	return NULL;
}


int rgw_work_start(void)
{
	int i;
	TRACE_ENTRY();
	
	memset(workers, 0, sizeof(workers));
	rg_list_init(&work_data);
	
	/* Create the worker thread(s) */
	for (i = 0; i < NB_WORKERS; i++) {
		CHECK_POSIX( pthread_create(&workers[i], NULL, work_th, NULL) );
	}
	
	return 0;
}

int rgw_work_add(rad_t * msg, void * client)
{
	struct work_item * new;
	
	CHECK_MALLOC( new = malloc(sizeof(struct work_item)) );
	memset(new, 0, sizeof(struct work_item));
	rg_list_init(&new->chain);
	
	new->msg = msg;
	new->cli = client;
	
	CHECK_POSIX( pthread_mutex_lock(&work_mtx) );
	rg_list_insert_before(&work_data, &new->chain);
	CHECK_POSIX( pthread_cond_signal(&work_cond) );
	CHECK_POSIX( pthread_mutex_unlock(&work_mtx) );
	
	return 0;
}

void rgw_work_fini(void)
{
	TRACE_ENTRY();
	
	/* kill all threads */
	
	return;
}
